@@ -1,9 +1,9 @@
 -- Copyright (c) 2020-2023, Shayne Fletcher. All rights reserved.
 -- SPDX-License-Identifier: BSD-3-Clause.
 
-{-# LANGUAGE CPP #-}
-{-# OPTIONS_GHC -Wno-orphans #-}
+{- ORMOLU_DISABLE -}
 #include "ghclib_api.h"
+{-# OPTIONS_GHC -Wno-orphans #-}
 module Language.Haskell.GhclibParserEx.GHC.Driver.Session(
       readExtension
     , extensionImplications
@@ -14,37 +14,45 @@ module Language.Haskell.GhclibParserEx.GHC.Driver.Session(
     , parsePragmasIntoDynFlags
   ) where
 
-#if defined (GHC_8_8) || defined (GHC_8_10)
+#if defined (GHC_8_8)
 import qualified GHC.LanguageExtensions as LangExt
-#endif
-#if defined (GHC_9_10) || defined (GHC_9_8) || defined (GHC_9_6) || defined (GHC_9_4) || defined(GHC_9_2) || defined (GHC_9_0)
-import GHC.Utils.Panic
-import GHC.Parser.Header
-import GHC.Data.StringBuffer
-import GHC.Driver.Session
-#if defined (GHC_9_10) || defined (GHC_9_8) || defined (GHC_9_6) || defined (GHC_9_4) || defined(GHC_9_2)
-import GHC.Types.SourceError
-#else
-import GHC.Driver.Types
-#endif
-#else
 import Panic
 import HeaderInfo
 import StringBuffer
 import DynFlags
 import HscTypes
+#elif defined (GHC_8_10)
+import qualified GHC.LanguageExtensions as LangExt
+import Panic
+import HeaderInfo
+import StringBuffer
+import DynFlags
+import HscTypes
+#elif defined (GHC_9_0)
+import GHC.Utils.Panic
+import GHC.Parser.Header
+import GHC.Data.StringBuffer
+import GHC.Driver.Session
+import GHC.Driver.Types
+#else
+import GHC.Utils.Panic
+import GHC.Parser.Header
+import GHC.Data.StringBuffer
+import GHC.Driver.Session
+import GHC.Types.SourceError
 #endif
 import GHC.LanguageExtensions.Type
 import Data.List
 import Data.Maybe
 import qualified Data.Map as Map
+
 -- Landed in https://gitlab.haskell.org/ghc/ghc/merge_requests/2707.
 #if defined (GHC_8_8) || defined (GHC_8_10)
 import Data.Function -- For `compareOn`.
 instance Ord Extension where
   compare = compare `on` fromEnum
 #endif
-#if defined (GHC_9_10) || defined (GHC_9_8) || defined (GHC_9_6) || defined (GHC_9_4)
+#if ! (defined (GHC_9_2) || defined (GHC_9_0) || defined (GHC_8_10) || defined(GHC_8_8) )
 import GHC.Driver.Config.Parser
 #endif
 
@@ -52,6 +60,7 @@ import GHC.Driver.Config.Parser
 readExtension :: String -> Maybe Extension
 readExtension s = flagSpecFlag <$> find (\(FlagSpec n _ _ _) -> n == s) xFlags
 
+#if (defined (GHC_9_12) || defined (GHC_9_10) || defined (GHC_9_8) || defined (GHC_9_6) || defined (GHC_9_4) || defined (GHC_9_2) || defined (GHC_9_0) || defined (GHC_8_10) || defined(GHC_8_8) )
 -- | Implicitly enabled/disabled extensions.
 extensionImplications :: [(Extension, ([Extension], [Extension]))]
 extensionImplications = map f $ Map.toList implicationsMap
@@ -60,7 +69,27 @@ extensionImplications = map f $ Map.toList implicationsMap
     implicationsMap :: Map.Map String ([Extension], [Extension])
     implicationsMap = Map.fromListWith (<>)
       [(show a, ([c | b], [c | not b]))
-        | (a, flag, c) <- impliedXFlags, let b = flag == turnOn]
+        | (a, flag, c) <- impliedXFlags,
+          let b = flag == turnOn
+      ]
+#else
+  {- defined (GHC_9_14) -}
+-- | Implicitly enabled/disabled extensions.
+extensionImplications :: [(Extension, ([Extension], [Extension]))]
+extensionImplications = map f $ Map.toList implicationsMap
+  where
+    f (e, ps) = (fromJust (readExtension e), ps)
+    implicationsMap :: Map.Map String ([Extension], [Extension])
+    implicationsMap = Map.fromListWith (<>)
+      [(show a, ([strip c | b], [strip c | not b]))
+        | (a, c) <- impliedXFlags,
+          let b = case c of On _ -> True; Off _ -> False
+      ]
+
+    strip :: OnOff a -> a
+    strip (On e) = e
+    strip (Off e) = e
+#endif
 
 -- Landed in
 -- https://gitlab.haskell.org/ghc/ghc/merge_requests/2654. Copied from
@@ -161,14 +190,13 @@ parsePragmasIntoDynFlags :: DynFlags
                          -> IO (Either String DynFlags)
 parsePragmasIntoDynFlags flags (enable, disable) file str =
   catchErrors $ do
-#if defined (GHC_9_10) || defined (GHC_9_8) || defined (GHC_9_6) || defined (GHC_9_4)
-    let (_, opts) =
-          getOptions (initParserOpts flags) (stringToStringBuffer str) file
+#if (defined (GHC_9_2) || defined (GHC_9_0) || defined (GHC_8_10) || defined (GHC_8_8))
+    let opts = getOptions flags (stringToStringBuffer str) file
+#elif (defined (GHC_9_12) || defined (GHC_9_10) || defined (GHC_9_8) || defined (GHC_9_6) || defined (GHC_9_4))
+    let (_, opts) = getOptions (initParserOpts flags) (stringToStringBuffer str) file
 #else
-    let opts =
-          getOptions flags (stringToStringBuffer str) file
+    let (_, opts) = getOptions (initParserOpts flags) (supportedLanguagePragmas flags) (stringToStringBuffer str) file
 #endif
-
     -- Important : apply enables, disables *before* parsing dynamic
     -- file pragmas.
     let flags' =  foldl' xopt_set flags enable
