@@ -1,8 +1,8 @@
--- Copyright (c) 2020, Shayne Fletcher. All rights reserved.
+-- Copyright (c) 2020-2023, Shayne Fletcher. All rights reserved.
 -- SPDX-License-Identifier: BSD-3-Clause.
 
+{- ORMOLU_DISABLE -}
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
-{-# LANGUAGE CPP #-}
 #include "ghclib_api.h"
 
 module Language.Haskell.GhclibParserEx.GHC.Hs.ExtendInstances (
@@ -18,14 +18,14 @@ where
 -- representations rather than the terms themselves, leads to
 -- identical results.
 
-#if defined (GHCLIB_API_HEAD) || defined (GHCLIB_API_904) || defined(GHCLIB_API_902) || defined (GHCLIB_API_900)
+#if !( defined (GHC_8_8) || defined (GHC_8_10))
 import GHC.Utils.Outputable
 #else
 import Outputable
 #endif
 import Data.Data
 import Data.Function
-import Language.Haskell.GhclibParserEx.Dump
+import Language.Haskell.GhclibParserEx.GHC.Hs.Dump
 
 newtype HsExtendInstances a =
   HsExtendInstances { unextendInstances :: a }
@@ -39,7 +39,7 @@ extendInstances = HsExtendInstances
 -- string representations.
 toStr :: Data a => HsExtendInstances a -> String
 toStr (HsExtendInstances e) =
-#if defined (GHCLIB_API_HEAD) || defined (GHCLIB_API_904) || defined (GHCLIB_API_902)
+#if !( defined (GHC_9_0) || defined (GHC_8_10) || defined (GHC_8_8) )
   showPprUnsafe $ showAstData BlankSrcSpan BlankEpAnnotations e
 #else
   showSDocUnsafe $ showAstData BlankSrcSpan e
@@ -57,7 +57,7 @@ astListEq as bs = length as == length bs && all (uncurry astEq) (zip as bs)
 -- Use 'ppr' for 'Show'.
 instance Outputable a => Show (HsExtendInstances a) where
   show (HsExtendInstances e) =
-#if defined(GHCLIB_API_HEAD) || defined(GHCLIB_API_902)
+#if !( defined (GHC_9_0) || defined (GHC_8_10) || defined (GHC_8_8) )
     showPprUnsafe $ ppr e
 #else
     showSDocUnsafe $ ppr e
