@@ -2,8 +2,9 @@
 -- SPDX-License-Identifier: BSD-3-Clause.
 
 module Language.Haskell.GhclibParserEx.GHC.Hs.Types
-  {-# DEPRECATED "Use Language.Haskell.GhclibParserEx.GHC.Hs.Type instead" #-} (
-  module Language.Haskell.GhclibParserEx.GHC.Hs.Type
-  ) where
+  {-# DEPRECATED "Use Language.Haskell.GhclibParserEx.GHC.Hs.Type instead" #-}
+  ( module Language.Haskell.GhclibParserEx.GHC.Hs.Type,
+  )
+where
 
 import Language.Haskell.GhclibParserEx.GHC.Hs.Type
