@@ -1,33 +1,36 @@
--- Copyright (c) 2020, Shayne Fletcher. All rights reserved.
+-- Copyright (c) 2020-2023, Shayne Fletcher. All rights reserved.
 -- SPDX-License-Identifier: BSD-3-Clause.
 
-{-# LANGUAGE CPP #-}
 #include "ghclib_api.h"
-
-module Language.Haskell.GhclibParserEx.GHC.Types.Name.Reader(
-   occNameStr, rdrNameStr, isSpecial, unqual, fromQual, isSymbolRdrName
- )
+module Language.Haskell.GhclibParserEx.GHC.Types.Name.Reader
+  ( occNameStr,
+    rdrNameStr,
+    isSpecial,
+    unqual,
+    fromQual,
+    isSymbolRdrName,
+  )
 where
 
-#if defined (GHCLIB_API_HEAD) || defined(GHCLIB_API_904) || defined (GHCLIB_API_902)
+#if !( defined (GHC_9_0) || defined (GHC_8_10) || defined (GHC_8_8) )
 import GHC.Parser.Annotation
 #endif
 
-#if defined (GHCLIB_API_HEAD) || defined(GHCLIB_API_904) || defined(GHCLIB_API_902) || defined (GHCLIB_API_900)
-import GHC.Types.SrcLoc
-import GHC.Types.Name
-import GHC.Types.Name.Reader
-#else
+#if defined (GHC_8_8) || defined (GHC_8_10)
 import SrcLoc
 import RdrName
 import OccName
 import Name
+#else
+import GHC.Types.SrcLoc
+import GHC.Types.Name
+import GHC.Types.Name.Reader
 #endif
 
 -- These names may not seem natural here but they work out in
 -- practice. The use of thse two functions is thoroughly ubiquitous.
 occNameStr :: RdrName -> String; occNameStr = occNameString . rdrNameOcc
-#if defined (GHCLIB_API_HEAD) || defined(GHCLIB_API_904) || defined (GHCLIB_API_902)
+#if ! ( defined (GHC_9_0) || defined (GHC_8_10) || defined (GHC_8_8) )
 rdrNameStr :: GHC.Parser.Annotation.LocatedN RdrName -> String
 #else
 rdrNameStr :: Located RdrName -> String
@@ -35,7 +38,7 @@ rdrNameStr :: Located RdrName -> String
 rdrNameStr = occNameStr . unLoc
 
 -- Builtin type or data constructors.
-#if defined (GHCLIB_API_HEAD) || defined(GHCLIB_API_904) || defined (GHCLIB_API_902)
+#if ! ( defined (GHC_9_0) || defined (GHC_8_10) || defined (GHC_8_8) )
 isSpecial :: LocatedN RdrName -> Bool
 #else
 isSpecial :: Located RdrName -> Bool
@@ -45,7 +48,7 @@ isSpecial _ = False
 
 -- Coerce qualified names to unqualified (by discarding the
 -- qualifier).
-#if defined (GHCLIB_API_HEAD) || defined(GHCLIB_API_904) || defined (GHCLIB_API_902)
+#if ! ( defined (GHC_9_0) || defined (GHC_8_10) || defined (GHC_8_8) )
 unqual :: LocatedN RdrName -> LocatedN RdrName
 #else
 unqual :: Located RdrName -> Located RdrName
@@ -54,7 +57,7 @@ unqual (L loc (Qual _ n)) = L loc $ mkRdrUnqual n
 unqual x = x
 
 -- Extract the occ name from a qualified/unqualified reader name.
-#if defined (GHCLIB_API_HEAD) || defined(GHCLIB_API_904) || defined (GHCLIB_API_902)
+#if ! ( defined (GHC_9_0) || defined (GHC_8_10) || defined (GHC_8_8) )
 fromQual :: LocatedN RdrName -> Maybe OccName
 #else
 fromQual :: Located RdrName -> Maybe OccName
