@@ -1,22 +1,22 @@
--- Copyright (c) 2020, Shayne Fletcher. All rights reserved.
+-- Copyright (c) 2020-2023, Shayne Fletcher. All rights reserved.
 -- SPDX-License-Identifier: BSD-3-Clause.
 
-{-# LANGUAGE CPP #-}
+{- ORMOLU_DISABLE -}
 #include "ghclib_api.h"
 module Language.Haskell.GhclibParserEx.GHC.Utils.Outputable (
     unsafePrettyPrint
 )
 where
 
-#if defined (GHCLIB_API_HEAD) || defined (GHCLIB_API_904) || defined(GHCLIB_API_902) || defined (GHCLIB_API_900)
-import GHC.Utils.Outputable
-#else
+#if defined (GHC_8_8) || defined (GHC_8_10)
 import Outputable
+#else
+import GHC.Utils.Outputable
 #endif
 
 unsafePrettyPrint :: Outputable a => a -> String
 unsafePrettyPrint =
-#if defined (GHCLIB_API_HEAD) || defined (GHCLIB_API_904)
+#if ! ( defined (GHC_9_2) || defined (GHC_9_0) || defined (GHC_8_10) || defined (GHC_8_8) )
   showPprUnsafe . ppr
 #else
   showSDocUnsafe . ppr
