@@ -14,18 +14,19 @@ import System.Environment
 import qualified System.FilePath as FilePath
 import System.IO.Extra
 import Control.Monad
+import Data.Data
 import Data.List.Extra
 import Data.Maybe
 import Data.Generics.Uniplate.Data
-#if defined (GHC_9_10) || defined (GHC_9_8) || defined (GHC_9_6) || defined (GHC_9_4) || defined (GHC_9_2)
+#if !(defined (GHC_9_0) || defined (GHC_8_10) || defined (GHC_8_8))
 import GHC.Data.Bag
-#if !defined (GHC_9_2)
+#  if !defined (GHC_9_2)
 import GHC.Driver.Errors.Types
-#endif
+#  endif
 import GHC.Types.Error
 #endif
 
-import Language.Haskell.GhclibParserEx.Dump
+import Language.Haskell.GhclibParserEx.GHC.Hs.Dump
 import Language.Haskell.GhclibParserEx.Fixity
 import Language.Haskell.GhclibParserEx.GHC.Settings.Config
 import Language.Haskell.GhclibParserEx.GHC.Parser
@@ -47,21 +48,26 @@ import Language.Haskell.GhclibParserEx.GHC.Driver.Flags()
 import Language.Haskell.GhclibParserEx.GHC.Driver.Session
 import Language.Haskell.GhclibParserEx.GHC.Types.Name.Reader
 
-#if defined (GHC_9_10) || defined (GHC_9_8) || defined (GHC_9_6) || defined (GHC_9_4) || defined (GHC_9_2) || defined (GHC_9_0) || defined (GHC_8_10)
+#if !defined (GHC_8_8)
+-- ghc >= 8.10
 import GHC.Hs
 #else
 import HsSyn
 #endif
-#if defined (GHC_9_10) || defined (GHC_9_8) || defined (GHC_9_6) || defined (GHC_9_4) || defined (GHC_9_2) || defined (GHC_9_0)
+#if !( defined (GHC_8_10) || defined (GHC_8_8) )
+-- ghc >= 9.0
 import GHC.Types.SrcLoc
 import GHC.Driver.Session
 import GHC.Parser.Lexer
-#  if !defined (GHC_9_10) && !defined (GHC_9_8) && !defined (GHC_9_6)
+#  if defined (GHC_9_4) || defined (GHC_9_2) || defined (GHC_9_0)
+--   9.0 <= ghc <= 9.4
 import GHC.Utils.Outputable
 #  endif
 #  if !defined (GHC_9_0)
+--   ghc >= 9.2
 import GHC.Driver.Ppr
-#    if !defined (GHC_9_10) && !defined (GHC_9_8) && !defined (GHC_9_6) && !defined (GHC_9_4)
+#    if defined (GHC_9_2)
+--     ghc = 9.2
 import GHC.Parser.Errors.Ppr
 #    endif
 #  endif
@@ -69,6 +75,7 @@ import GHC.Utils.Error
 import GHC.Types.Name.Reader
 import GHC.Types.Name.Occurrence
 #else
+-- ghc < 9.0
 import SrcLoc
 import DynFlags
 import Lexer
@@ -77,15 +84,17 @@ import ErrUtils
 import RdrName
 import OccName
 #endif
-import GHC.LanguageExtensions.Type
 #if defined (GHC_8_8)
+-- ghc = 8.8
 import Bag
 #endif
+import GHC.LanguageExtensions.Type
 
 basicDynFlags :: DynFlags
 basicDynFlags =
   defaultDynFlags fakeSettings
-#if !defined (GHC_9_10) && !defined (GHC_9_8) && !defined (GHC_9_6)
+#if defined (GHC_9_4) || defined (GHC_9_2) || defined (GHC_9_0) || defined (GHC_8_10) || defined (GHC_8_8)
+-- ghc <= 9.4
                                 fakeLlvmConfig
 #endif
 
@@ -101,6 +110,7 @@ tests = testGroup " All tests"
   , fixityTests
   , extendInstancesTests
   , expressionPredicateTests
+  , declarationPredicateTests
   , typePredicateTests
   , patternPredicateTests
   , dynFlagsTests
@@ -113,7 +123,8 @@ makeFile relPath contents = do
     writeFile relPath contents
     return relPath
 
-#if defined(GHC_9_10) || defined (GHC_9_8) || defined(GHC_9_6)
+#if !(defined (GHC_9_4) || defined (GHC_9_2) || defined (GHC_9_0) || defined (GHC_8_10) || defined (GHC_8_8))
+-- ghc >= 9.6
 report :: DynFlags -> Bag (MsgEnvelope GhcMessage) -> String
 report flags msgs = concat [ showSDoc flags msg | msg <- pprMsgEnvelopeBagWithLocDefault msgs ]
 #elif defined (GHC_9_4)
@@ -130,7 +141,8 @@ report flags msgs = concat [ showSDoc flags msg | msg <- pprErrMsgBagWithLoc msg
 chkParseResult :: DynFlags -> ParseResult a -> IO ()
 chkParseResult flags = \case
     POk s _ -> do
-#if defined (GHC_9_10) || defined (GHC_9_8) || defined (GHC_9_6) || defined (GHC_9_4)
+#if !(defined (GHC_9_2) || defined (GHC_9_0) || defined (GHC_8_10) || defined (GHC_8_8))
+-- ghc >= 9.4
       let (wrns, errs) = getPsMessages s
 #elif defined (GHC_9_2)
       let (wrns, errs) = getMessages s
@@ -138,7 +150,8 @@ chkParseResult flags = \case
       let (wrns, errs) = getMessages s flags
 #endif
       when (not (null errs) || not (null wrns)) $
-#if defined (GHC_9_10) || defined (GHC_9_8) || defined (GHC_9_6) || defined (GHC_9_4)
+#if !(defined (GHC_9_2) || defined (GHC_9_0) || defined (GHC_8_10) || defined (GHC_8_8))
+-- ghc >= 9.4
         assertFailure (
           report flags (getMessages (GhcPsMessage <$> wrns)) ++
           report flags (getMessages (GhcPsMessage <$> errs))
@@ -148,7 +161,8 @@ chkParseResult flags = \case
 #else
         assertFailure (report flags wrns ++ report flags errs)
 #endif
-#if defined (GHC_9_10) || defined (GHC_9_8) || defined (GHC_9_6) || defined (GHC_9_4)
+#if !(defined (GHC_9_2) || defined (GHC_9_0) || defined (GHC_8_10) || defined (GHC_8_8))
+-- ghc >= 9.4
     PFailed s -> assertFailure (report flags $ getMessages (GhcPsMessage <$> snd (getPsMessages s)))
 #elif defined (GHC_9_2)
     PFailed s -> assertFailure (report flags $ fmap pprError (snd (getMessages s)))
@@ -158,6 +172,9 @@ chkParseResult flags = \case
     PFailed _ loc err -> assertFailure (report flags $ unitBag $ mkPlainErrMsg flags loc err)
 #endif
 
+hasS :: (Data x, Data a) => (a -> Bool) -> x -> Bool
+hasS test = any test . universeBi
+
 parseTests :: TestTree
 parseTests = testGroup "Parse tests"
   [
@@ -239,6 +256,12 @@ exprTest s flags test =
         POk _ e -> test e
         _ -> assertFailure "parse error"
 
+declTest :: String -> DynFlags -> (LHsDecl GhcPs -> IO ()) -> IO ()
+declTest s flags test =
+      case parseDeclaration s flags of
+        POk _ e -> test e
+        _ -> assertFailure "parse error"
+
 stmtTest :: String -> DynFlags -> (LStmt GhcPs (LHsExpr GhcPs) -> IO ()) -> IO ()
 stmtTest s flags test =
       case parseStatement s flags of
@@ -263,7 +286,8 @@ fixityTests = testGroup "Fixity tests"
       exprTest "1 + 2 * 3" flags
         (\e ->
             assertBool "parse tree not affected" $
-#if defined (GHC_9_10) || defined (GHC_9_8) || defined (GHC_9_6) || defined (GHC_9_4) || defined (GHC_9_2)
+#if !(defined (GHC_9_0) || defined (GHC_8_10) || defined (GHC_8_8))
+-- ghc >= 9.2
               showSDocUnsafe (showAstData BlankSrcSpan BlankEpAnnotations e) /=
               showSDocUnsafe (showAstData BlankSrcSpan BlankEpAnnotations (applyFixities [] e))
 #else
@@ -275,7 +299,8 @@ fixityTests = testGroup "Fixity tests"
       case parseDeclaration "f (1 : 2 :[]) = 1" flags of
         POk _ d ->
           assertBool "parse tree not affected" $
-#if defined (GHC_9_10) || defined (GHC_9_8) || defined (GHC_9_6) || defined (GHC_9_4) || defined (GHC_9_2)
+#if !(defined (GHC_9_0) || defined (GHC_8_10) || defined (GHC_8_8))
+-- ghc >= 9.2
           showSDocUnsafe (showAstData BlankSrcSpan BlankEpAnnotations d) /=
           showSDocUnsafe (showAstData BlankSrcSpan BlankEpAnnotations (applyFixities [] d))
 #else
@@ -321,6 +346,17 @@ typePredicateTests = testGroup "Type predicate tests"
     test_with_exts exts s = typeTest s (flags exts)
     flags = foldl' xopt_set basicDynFlags
 
+declarationPredicateTests :: TestTree
+declarationPredicateTests = testGroup "Declaration predicate tests"
+  [
+    testCase "isStrictMatch" $ test "x = e" $ assert' . not . hasS isStrictMatch
+  , testCase "isStrictMatch" $ test "!x = e" $ assert' . hasS isStrictMatch
+  ]
+  where
+    assert' = assertBool ""
+    test s = declTest s (flags [ BangPatterns ])
+    flags = foldl' xopt_set basicDynFlags
+
 expressionPredicateTests :: TestTree
 expressionPredicateTests = testGroup "Expression predicate tests"
   [ testCase "isTag" $ test "foo" $ assert' . isTag "foo"
@@ -329,12 +365,20 @@ expressionPredicateTests = testGroup "Expression predicate tests"
   , testCase "isDot" $ test "f . g" $ \case L _ (OpApp _ _ op _) -> assert' $ isDot op; _ -> assertFailure "unexpected"
   , testCase "isReturn" $ test "return x" $ \case L _ (HsApp _ f _) -> assert' $ isReturn f; _ -> assertFailure "unexpected"
   , testCase "isReturn" $ test "pure x" $ \case L _ (HsApp _ f _) -> assert' $ isReturn f; _ -> assertFailure "unexpected"
-#if defined (GHC_9_10) || defined (GHC_9_8) || defined (GHC_9_6) || defined (GHC_9_4)
+#if !(defined (GHC_9_8) || defined (GHC_9_6) || defined (GHC_9_4) || defined (GHC_9_2) || defined (GHC_9_0) || defined (GHC_8_10) || defined (GHC_8_8))
+-- ghc >= 9.8
+  , testCase "isSection" $ test "(1 +)" $ \case L _ (HsPar _ x) -> assert' $ isSection x; _ -> assertFailure "unexpected"
+#elif !(defined (GHC_9_2) || defined (GHC_9_0) || defined (GHC_8_10) || defined (GHC_8_8))
+-- ghc >= 9.4
   , testCase "isSection" $ test "(1 +)" $ \case L _ (HsPar _ _ x _) -> assert' $ isSection x; _ -> assertFailure "unexpected"
 #else
   , testCase "isSection" $ test "(1 +)" $ \case L _ (HsPar _ x) -> assert' $ isSection x; _ -> assertFailure "unexpected"
 #endif
-#if defined (GHC_9_10) || defined (GHC_9_8) || defined (GHC_9_6) || defined (GHC_9_4)
+#if !(defined (GHC_9_8) || defined (GHC_9_6) || defined (GHC_9_4) || defined (GHC_9_2) || defined (GHC_9_0) || defined (GHC_8_10) || defined (GHC_8_8))
+-- ghc >= 9.8
+  , testCase "isSection" $ test "(+ 1)" $ \case L _ (HsPar _ x) -> assert' $ isSection x; _ -> assertFailure "unexpected"
+#elif !(defined (GHC_9_2) || defined (GHC_9_0) || defined (GHC_8_10) || defined (GHC_8_8))
+-- ghc >= 9.4
   , testCase "isSection" $ test "(+ 1)" $ \case L _ (HsPar _ _ x _) -> assert' $ isSection x; _ -> assertFailure "unexpected"
 #else
   , testCase "isSection" $ test "(+ 1)" $ \case L _ (HsPar _ x) -> assert' $ isSection x; _ -> assertFailure "unexpected"
@@ -359,6 +403,7 @@ expressionPredicateTests = testGroup "Expression predicate tests"
   , testCase "isLexeme" $ test "3" $ assert' . isLexeme
   , testCase "isLexeme" $ test "f x" $ assert' . not . isLexeme
   , testCase "isLambda" $ test "\\x -> 12" $ assert' . isLambda
+  , testCase "isLambda" $ test_with_exts [ LambdaCase ]  "\\case _ -> 12" $ assert' . not . isLambda
   , testCase "isLambda" $ test "foo" $ assert' . not . isLambda
   , testCase "isDotApp" $ test "f . g" $ assert' . isDotApp
   , testCase "isDotApp" $ test "f $ g" $ assert' . not . isDotApp
@@ -389,7 +434,7 @@ expressionPredicateTests = testGroup "Expression predicate tests"
   , testCase "isSpliceDecl" $ test "$x" $ assert' . isSpliceDecl . unLoc
   , testCase "isSpliceDecl" $ test "f$x" $ assert' . not . isSpliceDecl . unLoc
   , testCase "isSpliceDecl" $ test "$(a + b)" $ assert' . isSpliceDecl . unLoc
-#if !( defined(GHC_8_8) || defined(GHC_8_10) || defined (GHC_9_0) || defined (GHC_9_2) || defined(GHC_9_4) )
+#if !(defined (GHC_9_4) || defined (GHC_9_2) || defined (GHC_9_0) || defined (GHC_8_10) || defined (GHC_8_8))
   -- ghc api >= 9.6.1
   , testCase "isTypedSplice" $ test "$$foo" $ assert' . isTypedSplice . unLoc
   , testCase "isTypedSplice" $ test "$foo" $ assert' . not . isTypedSplice . unLoc
@@ -443,6 +488,8 @@ patternPredicateTests = testGroup "Pattern predicate tests"
   , testCase "fromPChar" $ test "'a'" $ assert' . (== Just 'a') . fromPChar
   , testCase "fromPChar" $ test "\"a\"" $ assert' . isNothing . fromPChar
   , testCase "isSplicePat" $ test "$(varP pylonExPtrVarName)" $ assert' . isSplicePat
+  , testCase "isWildPat" $ test "_" $ assert' . isWildPat
+  , testCase "isWildPat" $ test "p@(L _ (VisPat _ pat))" $ assert' . not . isWildPat
   ]
   where
     assert' = assertBool ""
